from django import forms


class SemanticSelect(forms.Select):
    template_name = "semantic_ui/forms/widgets/select.html"


class SemanticSelectMultiple(forms.SelectMultiple, SemanticSelect):
    pass


class SemanticCheckboxInput(forms.CheckboxInput):
    template_name = "semantic_ui/forms/widgets/checkbox.html"


class SemanticCheckboxSelectMultiple(forms.CheckboxSelectMultiple):
    template_name = "semantic_ui/forms/widgets/checkbox_select.html"
    option_template_name = "semantic_ui/forms/widgets/checkbox_option.html"


class SemanticDateTimeInput(forms.DateTimeInput):
    template_name = "semantic_ui/forms/widgets/datetime.html"


class SemanticDateInput(forms.DateInput):
    template_name = "semantic_ui/forms/widgets/date.html"


class SemanticTimeInput(forms.TimeInput):
    template_name = "semantic_ui/forms/widgets/time.html"


class SemanticEmailInput(forms.EmailInput):
    # TODO
    # template_name = "semantic_ui/forms/widgets/email.html"
    pass


class SemanticFileInput(forms.ClearableFileInput):
    template_name = "semantic_ui/forms/widgets/clearable_file_input.html"


class SemanticImageInput(SemanticFileInput):
    pass


class SemanticClearableFileInput(forms.ClearableFileInput):
    template_name = "semantic_ui/forms/widgets/clearable_file_input.html"


class SemanticNumberInput(forms.NumberInput):
    pass


class SemanticPasswordInput(forms.NumberInput):
    pass


class SemanticRadioSelect(forms.RadioSelect):
    template_name = "semantic_ui/forms/widgets/radio.html"
    option_template_name = "semantic_ui/forms/widgets/radio_option.html"


class SemanticTextarea(forms.Textarea):
    template_name = "semantic_ui/forms/widgets/textarea.html"


class SemanticTextInput(forms.Textarea):
    template_name = "semantic_ui/forms/widgets/text.html"


class SemanticURLInput(forms.URLInput):
    pass


class RangeWidget(forms.MultiWidget):
    def decompress(self, value):
        if value:
            return [value.start, value.stop]
        return [None, None]


class SemanticDateRangeWidget(RangeWidget):
    def __init__(self, attrs=None):
        widgets = [SemanticDateInput, SemanticDateInput]
        super().__init__(widgets, attrs)


class SemanticTimeRangeWidget(RangeWidget):
    def __init__(self, attrs=None):
        widgets = [SemanticTimeInput(), SemanticTimeInput()]
        super().__init__(widgets, attrs)



from django import template
from django.conf import settings
from django.contrib import admin
from django.contrib.admin import AdminSite
from django.urls import resolve, reverse

register = template.Library()


def get_semantic_sidebar(app_list, current_app):
    semantic_sidebar = getattr(settings, "SEMANTIC_SIDEBAR", None)
    if semantic_sidebar:
        ordered = []
        for app_label in semantic_sidebar:
            for app in app_list:
                is_active = app["app_label"] == current_app
                app["is_active"] = is_active
                if app_label == app["app_label"]:
                    ordered.append(app)
        has_active = any([app["is_active"] for app in ordered])
        if len(ordered) and not has_active:
            ordered[0]["is_active"] = True
        app_list = ordered
    return app_list


def get_app_label(resolver_match):
    if "app_label" in resolver_match.kwargs:
        return resolver_match.kwargs.get("app_label")
    else:
        # Reconstruct from url_name.
        url_name = resolver_match.url_name
        # Exclude model and action.
        parts = url_name.split("_")[:-2]
        # Return parts.
        return "_".join(parts)


@register.simple_tag(takes_context=True)
def get_app_list(context):
    request = context["request"]
    resolver_match = resolve(request.path_info)
    admin_name = resolver_match.namespace
    current_app = get_app_label(resolver_match)
    admin_site = get_admin_site(admin_name)
    app_list = admin_site.get_app_list(request)
    return get_semantic_sidebar(app_list, current_app)


def get_admin_site(current_app):
    try:
        resolver_match = resolve(reverse("%s:index" % current_app))
        for func_closure in resolver_match.func.func_closure:
            if isinstance(func_closure.cell_contents, AdminSite):
                return func_closure.cell_contents
    except Exception:
        pass
    return admin.site


def get_admin_url(request, admin_site):
    try:
        url = "{}:index".format(admin_site)
        url = reverse(url)
    except Exception:
        pass
    else:
        return url


@register.simple_tag(takes_context=True)
def admin_apps(context):
    return get_app_list(context)



from django.contrib.admin.helpers import ActionForm

from .fields import SemanticChoiceField

try:
    from django.utils.translation import gettext_lazy as _  # Django >= 4
except ImportError:
    from django.utils.translation import ugettext_lazy as _


class SemanticActionForm(ActionForm):
    action = SemanticChoiceField(label=_("Action:"))



from django import forms


class SemanticChangelistCheckboxInput(forms.CheckboxInput):
    template_name = "semantic_ui/forms/widgets/changelist_checkbox.html"


class SemanticActionCheckboxInput(forms.CheckboxInput):
    template_name = "semantic_ui/forms/widgets/action_checkbox.html"



from django.contrib.admin.views.autocomplete import AutocompleteJsonView
from django.core.exceptions import PermissionDenied
from django.http import JsonResponse


class SemanticAutocompleteJsonView(AutocompleteJsonView):
    # TODO: Delete overridden get method once serialize_result is released
    # in a future Django version
    def get(self, request, *args, **kwargs):
        """
        Return a JsonResponse with search results of the form:
        {
            results: [{id: "123" text: "foo"}],
            pagination: {more: true}
        }
        """
        (
            self.term,
            self.model_admin,
            self.source_field,
            to_field_name,
        ) = self.process_request(request)

        if not self.has_perm(request):
            raise PermissionDenied

        self.object_list = self.get_queryset()
        context = self.get_context_data()
        return JsonResponse(
            {
                # BEGIN CUSTOMIZATION #
                "results": [
                    self.serialize_result(obj, to_field_name)
                    for obj in context["object_list"]
                ],
                # END CUSTOMIZATION #
                "pagination": {"more": context["page_obj"].has_next()},
            }
        )

    def serialize_result(self, obj, to_field_name):
        """
        Convert the provided model object to a dictionary that is added to the
        results list.
        """
        return {
            "id": str(getattr(obj, to_field_name)),
            "name": getattr(obj, "semantic_autocomplete", str(obj)),
            "text": str(obj),
        }



from django import forms

from .widgets import (
    SemanticDateInput,
    SemanticDateTimeInput,
    SemanticSelect,
    SemanticSelectMultiple,
    SemanticTimeInput,
)


class SemanticDateTimeField(forms.DateTimeField):
    widget = SemanticDateTimeInput


class SemanticDateField(forms.DateField):
    widget = SemanticDateInput


class SemanticTimeField(forms.TimeField):
    widget = SemanticTimeInput


# Single choice
class SemanticChoiceField(forms.ChoiceField):
    widget = SemanticSelect


class SemanticTypedChoiceField(forms.TypedChoiceField):
    widget = SemanticSelect


class SemanticModelChoiceField(forms.ModelChoiceField):
    widget = SemanticSelect


# Multiple choice
class SemanticMultipleChoiceField(forms.MultipleChoiceField):
    widget = SemanticSelectMultiple


class SemanticTypedMultipleChoiceField(forms.TypedMultipleChoiceField):
    widget = SemanticSelectMultiple


class SemanticModelMultipleChoiceField(forms.ModelMultipleChoiceField):
    widget = SemanticSelectMultiple



from django_filters.filters import (
    AllValuesMultipleFilter,
    ChoiceFilter,
    DateFilter,
    DateTimeFilter,
    Filter,
    ModelChoiceFilter,
    ModelMultipleChoiceFilter,
    MultipleChoiceFilter,
    TimeFilter,
    TypedMultipleChoiceFilter,
)
from semantic_admin.fields import (
    SemanticChoiceField,
    SemanticDateField,
    SemanticDateTimeField,
    SemanticModelChoiceField,
    SemanticModelMultipleChoiceField,
    SemanticMultipleChoiceField,
    SemanticTimeField,
    SemanticTypedChoiceField,
    SemanticTypedMultipleChoiceField,
)


class FilterOrExcludeMixin(Filter):
    def get_method(self, qs):
        """Return filter method based on whether we're excluding or simply filtering"""
        return qs.exclude if self.exclude else qs.filter


class SemanticDateTimeFilter(FilterOrExcludeMixin, DateTimeFilter):
    field_class = SemanticDateTimeField


class SemanticDateFilter(FilterOrExcludeMixin, DateFilter):
    field_class = SemanticDateField


class SemanticTimeFilter(FilterOrExcludeMixin, TimeFilter):
    field_class = SemanticTimeField


class SemanticChoiceFilter(FilterOrExcludeMixin, ChoiceFilter):
    field_class = SemanticChoiceField


class SemanticMultipleChoiceFilter(FilterOrExcludeMixin, MultipleChoiceFilter):
    field_class = SemanticMultipleChoiceField


class SemanticTypedChoiceFilter(FilterOrExcludeMixin, TypedMultipleChoiceFilter):
    field_class = SemanticTypedChoiceField


class SemanticTypedMultipleChoiceFilter(
    FilterOrExcludeMixin, TypedMultipleChoiceFilter
):
    field_class = SemanticTypedMultipleChoiceField


class SemanticMultipleAllValuesFilter(FilterOrExcludeMixin, AllValuesMultipleFilter):
    field_class = SemanticChoiceField


class SemanticModelChoiceFilter(FilterOrExcludeMixin, ModelChoiceFilter):
    field_class = SemanticModelChoiceField


class SemanticModelMultipleChoiceFilter(
    FilterOrExcludeMixin, ModelMultipleChoiceFilter
):
    field_class = SemanticModelMultipleChoiceField



import datetime

from django import template
from django.contrib.admin.templatetags.admin_list import ResultList, _coerce_field_name
from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
from django.contrib.admin.utils import lookup_field
from django.contrib.admin.widgets import RelatedFieldWidgetWrapper
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.urls import NoReverseMatch
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from semantic_admin.utils import semantic_display_for_field, semantic_display_for_value

register = template.Library()


def _semantic_boolean_icon(field_val):
    return format_html(
        '<i class="%s circle icon"></i>'
        % {True: "green check", False: "red times", None: "gray question"}[field_val]
    )


def semantic_items_for_result(cl, result, form):
    """
    Generate the actual list of data.
    """

    def link_in_col(is_first, field_name, cl):
        if cl.list_display_links is None:
            return False
        if is_first and not cl.list_display_links:
            return True
        return field_name in cl.list_display_links

    first = True
    pk = cl.lookup_opts.pk.attname
    for field_index, field_name in enumerate(cl.list_display):
        empty_value_display = cl.model_admin.get_empty_value_display()
        row_classes = ["field-%s" % _coerce_field_name(field_name, field_index)]
        try:
            f, attr, value = lookup_field(field_name, result, cl.model_admin)
        except ObjectDoesNotExist:
            result_repr = empty_value_display
        else:
            empty_value_display = getattr(
                attr, "empty_value_display", empty_value_display
            )
            if f is None or f.auto_created:
                if field_name == "action_checkbox":
                    row_classes = ["action-checkbox"]
                boolean = getattr(attr, "boolean", False)
                # BEGIN CUSTOMIZATION
                result_repr = semantic_display_for_value(
                    value, empty_value_display, boolean
                )
                # END CUSTOMIZATION
                if isinstance(value, (datetime.date, datetime.time)):
                    row_classes.append("nowrap")
            else:
                if isinstance(f.remote_field, models.ManyToOneRel):
                    field_val = getattr(result, f.name)
                    if field_val is None:
                        result_repr = empty_value_display
                    else:
                        result_repr = field_val
                else:
                    # BEGIN CUSTOMIZATION
                    result_repr = semantic_display_for_field(
                        value, f, empty_value_display
                    )
                    # END CUSTOMIZATION
                if isinstance(
                    f, (models.DateField, models.TimeField, models.ForeignKey)
                ):
                    row_classes.append("nowrap")
        row_class = mark_safe(' class="%s"' % " ".join(row_classes))
        # If list_display_links not defined, add the link tag to the first field
        if link_in_col(first, field_name, cl):

            # BEGIN CUSTOMIZATION
            table_tag = "td"
            # END CUSTOMIZATION

            first = False

            # Display link to the result's change_view if the url exists, else
            # display just the result's representation.
            try:
                url = cl.url_for_result(result)
            except NoReverseMatch:
                link_or_text = result_repr
            else:
                url = add_preserved_filters(
                    {"preserved_filters": cl.preserved_filters, "opts": cl.opts}, url
                )
                # Convert the pk to something that can be used in Javascript.
                # Problem cases are non-ASCII strings.
                if cl.to_field:
                    attr = str(cl.to_field)
                else:
                    attr = pk
                value = result.serializable_value(attr)
                link_or_text = format_html(
                    '<a href="{}"{}>{}</a>',
                    url,
                    format_html(' data-popup-opener="{}"', value)
                    if cl.is_popup
                    else "",
                    result_repr,
                )

            yield format_html(
                "<{}{}>{}</{}>", table_tag, row_class, link_or_text, table_tag
            )
        else:
            # By default the fields come from ModelAdmin.list_editable, but if we pull
            # the fields out of the form instead of list_editable custom admins
            # can provide fields on a per request basis
            if (
                form
                and field_name in form.fields
                and not (
                    field_name == cl.model._meta.pk.name
                    and form[cl.model._meta.pk.name].is_hidden
                )
            ):

                # BEGIN CUSTOMIZATION
                field = form.fields[field_name]
                bf = form[field_name]
                if isinstance(field.widget, RelatedFieldWidgetWrapper):
                    bf_repr = str(bf.errors) + "&nbsp;" + str(bf)
                else:
                    bf_repr = str(bf.errors) + str(bf)
                result_repr = mark_safe(bf_repr)
                # END CUSTOMIZATION

            yield format_html("<td{}>{}</td>", row_class, result_repr)
    if form and not form[cl.model._meta.pk.name].is_hidden:
        yield format_html("<td>{}</td>", form[cl.model._meta.pk.name])


def semantic_results(cl):
    if cl.formset:
        for res, form in zip(cl.result_list, cl.formset.forms):
            yield ResultList(form, semantic_items_for_result(cl, res, form))
    else:
        for res in cl.result_list:
            yield ResultList(None, semantic_items_for_result(cl, res, None))



from .admin import (
    SemanticModelAdmin,
    SemanticStackedInline,
    SemanticTabularInline,
)

__all__ = [
    'SemanticModelAdmin',
    'SemanticStackedInline',
    'SemanticTabularInline',
]



import copy
from functools import partial

from django import forms
from django.contrib.admin import helpers, widgets
from django.contrib.admin.options import (
    BaseModelAdmin,
    StackedInline,
    TabularInline,
    get_ul_class,
)
from django.db import models
from django.forms.models import (
    modelform_defines_fields,
    modelform_factory,
    modelformset_factory,
)
from django.utils.html import format_html

from semantic_admin.widgets import (
    SemanticActionCheckboxInput,
    SemanticAutocompleteSelect,
    SemanticAutocompleteSelectMultiple,
    SemanticChangelistCheckboxInput,
    SemanticCheckboxInput,
    SemanticDateInput,
    SemanticDateTimeInput,
    SemanticEmailInput,
    SemanticFileInput,
    SemanticImageInput,
    SemanticNumberInput,
    SemanticRadioSelect,
    SemanticSelect,
    SemanticSelectMultiple,
    SemanticTextarea,
    SemanticTextInput,
    SemanticTimeInput,
    SemanticURLInput,
)

from .awesomesearch import AwesomeSearchModelAdmin
from .helpers import SemanticActionForm
from .views.autocomplete import SemanticAutocompleteJsonView

try:
    from django.utils.translation import ugettext_lazy as _
except ImportError:
    from django.utils.translation import gettext_lazy as _

SEMANTIC_FORMFIELD_FOR_DBFIELD_DEFAULTS = {
    models.DateTimeField: {"widget": SemanticDateTimeInput},
    models.DateField: {"widget": SemanticDateInput},
    models.TimeField: {"widget": SemanticTimeInput},
    models.TextField: {"widget": SemanticTextarea},
    models.URLField: {"widget": SemanticURLInput},
    # TODO
    models.IntegerField: {"widget": SemanticNumberInput},
    models.BigIntegerField: {"widget": SemanticNumberInput},
    # END TODO
    models.CharField: {"widget": SemanticTextInput},
    models.ImageField: {"widget": SemanticImageInput},
    models.FileField: {"widget": SemanticFileInput},
    models.EmailField: {"widget": SemanticEmailInput},
    models.UUIDField: {"widget": SemanticTextInput},
    models.BooleanField: {"widget": SemanticCheckboxInput},
}


class SemanticBaseModelAdmin(BaseModelAdmin):  # type: ignore
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Simply overwrite
        overrides = copy.deepcopy(SEMANTIC_FORMFIELD_FOR_DBFIELD_DEFAULTS)
        for k, v in overrides.items():
            self.formfield_overrides.setdefault(k, {}).update(v)
        self.formfield_overrides = overrides

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        """
        Get a form Field for a database Field that has declared choices.
        """
        # If the field is named as a radio_field, use a RadioSelect
        if db_field.name in self.radio_fields:
            # Avoid stomping on custom widget/choices arguments.
            if "widget" not in kwargs:

                # BEGIN CUSTOMIZATION
                kwargs["widget"] = SemanticRadioSelect(
                    attrs={
                        "class": get_ul_class(self.radio_fields[db_field.name]),
                    }
                )
                # END CUSTOMIZATION

            if "choices" not in kwargs:
                kwargs["choices"] = db_field.get_choices(
                    include_blank=db_field.blank, blank_choice=[("", _("None"))]
                )

        # BEGIN CUSTOMIZATION
        if "widget" not in kwargs:
            kwargs["widget"] = SemanticSelect()
        # END CUSTOMIZATION

        return db_field.formfield(**kwargs)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        """
        Get a form Field for a ForeignKey.
        """
        db = kwargs.get("using")

        if db_field.name in self.get_autocomplete_fields(request):
            # BEGIN CUSTOMIZATION

            kwargs["widget"] = SemanticAutocompleteSelect(
                db_field, self.admin_site, using=db
            )

            # END CUSTOMIZATION

        elif db_field.name in self.raw_id_fields:
            # TODO
            kwargs["widget"] = widgets.ForeignKeyRawIdWidget(
                db_field.remote_field, self.admin_site, using=db
            )
        elif db_field.name in self.radio_fields:
            kwargs["widget"] = SemanticRadioSelect(
                attrs={"class": get_ul_class(self.radio_fields[db_field.name])}
            )
            kwargs["empty_label"] = _("None") if db_field.blank else None

        if "queryset" not in kwargs:
            queryset = self.get_field_queryset(db, db_field, request)
            if queryset is not None:
                kwargs["queryset"] = queryset

        # BEGIN CUSTOMIZATION
        if "widget" not in kwargs:
            kwargs["widget"] = SemanticSelect()
        # END CUSTOMIZATION

        return db_field.formfield(**kwargs)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        """
        Get a form Field for a ManyToManyField.
        """
        # If it uses an intermediary model that isn't auto created, don't show
        # a field in admin.
        if not db_field.remote_field.through._meta.auto_created:
            return None
        db = kwargs.get("using")

        if "widget" not in kwargs:
            autocomplete_fields = self.get_autocomplete_fields(request)
            if db_field.name in autocomplete_fields:

                # BEGIN CUSTOMIZATION
                kwargs["widget"] = SemanticAutocompleteSelectMultiple(
                    db_field,
                    self.admin_site,
                    using=db,
                )
                # END CUSTOMIZATION

            elif db_field.name in self.raw_id_fields:
                kwargs["widget"] = widgets.ManyToManyRawIdWidget(
                    db_field.remote_field,
                    self.admin_site,
                    using=db,
                )
            elif db_field.name in [*self.filter_vertical, *self.filter_horizontal]:
                # TODO
                kwargs["widget"] = widgets.FilteredSelectMultiple(
                    db_field.verbose_name, db_field.name in self.filter_vertical
                )
        if "queryset" not in kwargs:
            queryset = self.get_field_queryset(db, db_field, request)
            if queryset is not None:
                kwargs["queryset"] = queryset

        # BEGIN CUSTOMIZATION
        if "widget" not in kwargs:
            kwargs["widget"] = SemanticSelectMultiple()
        # END CUSTOMIZATION

        form_field = db_field.formfield(**kwargs)

        # BEGIN CUSTOMIZATION
        # if isinstance(form_field.widget, SemanticSelectMultiple) and not isinstance(
        #     form_field.widget,
        #     (SemanticCheckboxSelectMultiple, SemanticAutocompleteSelectMultiple),
        # ):
        #     msg = _(
        #         "Hold down “Control”, or “Command” on a Mac, to select more than one."
        #     )
        #     help_text = form_field.help_text
        #     form_field.help_text = (
        #         format_lazy("{} {}", help_text, msg) if help_text else msg
        #     )
        # END CUSTOMIZATION

        return form_field

    def get_changelist_form(self, request, **kwargs):
        """
        Return a Form class for use in the Formset on the changelist page.
        """
        # BEGIN CUSTOMIZATION
        defaults = {
            "formfield_callback": partial(
                # Override formfield_callback so checkboxes render correctly
                self.changelist_formfield_for_dbfield,
                request=request,
            ),
            **kwargs,
        }
        # END CUSTOMIZATION
        if defaults.get("fields") is None and not modelform_defines_fields(
            defaults.get("form")
        ):
            defaults["fields"] = forms.ALL_FIELDS
        return modelform_factory(self.model, **defaults)

    def get_changelist_formset(self, request, **kwargs):
        """
        Return a FormSet class for use on the changelist page if list_editable
        is used.
        """
        # BEGIN CUSTOMIZATION
        defaults = {
            # Override formfield_callback so checkboxes render correctly
            "formfield_callback": partial(
                self.changelist_formfield_for_dbfield, request=request
            ),
            **kwargs,
        }
        # END CUSTOMIZATION
        return modelformset_factory(
            self.model,
            self.get_changelist_form(request),
            extra=0,
            fields=self.list_editable,
            **defaults
        )

    def changelist_formfield_for_dbfield(self, db_field, request, **kwargs):
        formfield = super().formfield_for_dbfield(db_field, request, **kwargs)
        if formfield and isinstance(formfield.widget, SemanticCheckboxInput):
            formfield.widget = SemanticChangelistCheckboxInput()
        return formfield

    def get_exclude(self, request, obj=None):
        # TODO: Verify and delete this method
        """
        Hook for specifying exclude.
        """
        return self.exclude


class SemanticModelAdmin(SemanticBaseModelAdmin, AwesomeSearchModelAdmin):
    action_form = SemanticActionForm

    def action_checkbox(self, obj):
        """
        A list_display column containing a checkbox widget.
        """
        semantic_checkbox = SemanticActionCheckboxInput(
            {"class": "action-select"}, lambda value: False
        )
        return semantic_checkbox.render(helpers.ACTION_CHECKBOX_NAME, str(obj.pk))

    action_checkbox.short_description = format_html(  # type: ignore
        format_html(
            """
            <div id="action-toggle" class="ui checkbox">
                <label></label><input id="action-toggle-input" type="checkbox">
            </div>
            """
        )
    )

    def autocomplete_view(self, request):
        return SemanticAutocompleteJsonView.as_view(model_admin=self)(request)


class SemanticStackedInline(SemanticBaseModelAdmin, StackedInline):
    pass


class SemanticTabularInline(SemanticBaseModelAdmin, TabularInline):
    pass



from .admin import SemanticActionCheckboxInput, SemanticChangelistCheckboxInput
from .autocomplete import SemanticAutocompleteSelect, SemanticAutocompleteSelectMultiple
from .widgets import (
    SemanticCheckboxInput,
    SemanticCheckboxSelectMultiple,
    SemanticClearableFileInput,
    SemanticDateInput,
    SemanticDateRangeWidget,
    SemanticDateTimeInput,
    SemanticEmailInput,
    SemanticFileInput,
    SemanticImageInput,
    SemanticNumberInput,
    SemanticPasswordInput,
    SemanticRadioSelect,
    SemanticSelect,
    SemanticSelectMultiple,
    SemanticTextarea,
    SemanticTextInput,
    SemanticTimeInput,
    SemanticTimeRangeWidget,
    SemanticURLInput,
)

__all__ = [
    "SemanticActionCheckboxInput",
    "SemanticAutocompleteSelect",
    "SemanticAutocompleteSelectMultiple",
    "SemanticChangelistCheckboxInput",
    "SemanticCheckboxInput",
    "SemanticCheckboxSelectMultiple",
    "SemanticClearableFileInput",
    "SemanticDateInput",
    "SemanticDateRangeWidget",
    "SemanticDateTimeInput",
    "SemanticEmailInput",
    "SemanticFileInput",
    "SemanticImageInput",
    "SemanticNumberInput",
    "SemanticPasswordInput",
    "SemanticRadioSelect",
    "SemanticSelect",
    "SemanticSelectMultiple",
    "SemanticTextarea",
    "SemanticTextInput",
    "SemanticTimeInput",
    "SemanticTimeRangeWidget",
    "SemanticURLInput",
]



from copy import deepcopy

from django.db import models
from django.db.models.fields.related import ManyToManyRel, ManyToOneRel, OneToOneRel
from django_filters import FilterSet
from django_filters.filters import BaseInFilter, BaseRangeFilter
from django_filters.filterset import FILTER_FOR_DBFIELD_DEFAULTS
from django_filters.utils import try_dbfield

from .filters import (
    SemanticChoiceFilter,
    SemanticModelChoiceFilter,
    SemanticModelMultipleChoiceFilter,
)

SEMANTIC_FILTER_FOR_DBFIELD_DEFAULTS = {
    # Forward relationships
    models.OneToOneField: SemanticModelChoiceFilter,
    models.ForeignKey: SemanticModelChoiceFilter,
    models.ManyToManyField: SemanticModelMultipleChoiceFilter,
    # Reverse relationships
    OneToOneRel: SemanticModelChoiceFilter,
    ManyToOneRel: SemanticModelMultipleChoiceFilter,
    ManyToManyRel: SemanticModelMultipleChoiceFilter,
}


class SemanticFilterSet(FilterSet):
    FILTER_DEFAULTS = deepcopy(FILTER_FOR_DBFIELD_DEFAULTS)

    for field, filter_class in SEMANTIC_FILTER_FOR_DBFIELD_DEFAULTS.items():
        FILTER_DEFAULTS[field]["filter_class"] = filter_class

    def __init__(self, *args, **kwargs):
        self.passed_validation = kwargs.pop("passed_validation", False)
        super().__init__(*args, **kwargs)

    @classmethod
    def filter_for_lookup(cls, field, lookup_type):
        DEFAULTS = dict(cls.FILTER_DEFAULTS)
        if hasattr(cls, "_meta"):
            DEFAULTS.update(cls._meta.filter_overrides)

        data = try_dbfield(DEFAULTS.get, field.__class__) or {}
        filter_class = data.get("filter_class")
        params = data.get("extra", lambda field: {})(field)

        # if there is no filter class, exit early
        if not filter_class:
            return None, {}

        # perform lookup specific checks
        if lookup_type == "exact" and getattr(field, "choices", None):

            # BEGIN CUSTOMIZATION
            return SemanticChoiceFilter, {"choices": field.choices}
            # END CUSTOMIZATION

        if lookup_type == "isnull":
            data = try_dbfield(DEFAULTS.get, models.BooleanField)

            filter_class = data.get("filter_class")
            params = data.get("extra", lambda field: {})(field)
            return filter_class, params

        if lookup_type == "in":

            class ConcreteInFilter(BaseInFilter, filter_class):
                pass

            ConcreteInFilter.__name__ = cls._csv_filter_class_name(
                filter_class, lookup_type
            )

            return ConcreteInFilter, params

        if lookup_type == "range":

            class ConcreteRangeFilter(BaseRangeFilter, filter_class):
                pass

            ConcreteRangeFilter.__name__ = cls._csv_filter_class_name(
                filter_class, lookup_type
            )

            return ConcreteRangeFilter, params

        return filter_class, params


class SemanticExcludeAllFilterSet(SemanticFilterSet):
    """
    Debatable to have this
    https://django-filter.readthedocs.io/en/master/guide/tips.html
    """

    def __init__(self, *args, **kwargs):
        self.exclude = kwargs.pop("exclude", False)
        super().__init__(*args, **kwargs)
        if self.exclude:
            for f in self.filters:
                exclude = self.filters[f].exclude
                self.filters[f].exclude = not exclude



from django import forms, template
from django.utils.html import format_html
from django.utils.safestring import mark_safe

BLANK_LABEL = "<label>&nbsp;</label>"
FIELD = '<div class="field">{}</div>'
COMPUTER_FIELD = '<div class="computer only field"></div>'

register = template.Library()

try:
    from django.utils.translation import gettext_lazy as _  # Django >= 4
except ImportError:
    from django.utils.translation import ugettext_lazy as _


def format_fields(cl, fields):
    html = ""
    row = '<div class="equal width fields">{}</div>'
    r = []
    for index, field in enumerate(fields):
        r.append(field)
        i = index + 1
        is_divisible_by_4 = not i % 4
        if is_divisible_by_4:
            html += row.format("".join(r))
            r = []
    # Remaining fields.
    while len(r) < 3:
        r.append(COMPUTER_FIELD)
    search_button = format_search_button(cl)
    r.append(search_button)
    html += row.format("".join(r))
    return html


def format_search_field(context, cl):
    field = ""
    if len(cl.search_fields):
        label = _("Search")
        search_var = context["search_var"]
        search_label = f'<label for="searchbar">{label}: </label>'
        search_input = f"""
            <input
                id="searchbar"
                type="text"
                name="{search_var}"
                value="{cl.query}"
            />
        """
        if hasattr(cl.model_admin, "filterset"):
            field = f"{search_label}{search_input}"
        else:
            field = f"""
                <div class="ui action input">
                    {search_input}
                    <button class="ui blue button" type="submit">
                        <i class="search icon"></i>{label}
                    </button>
                </div>
            """
        return FIELD.format(field)
    else:
        return ""


def format_search_button(cl):
    html = ""
    search_label = _("Search")
    search_button = f"""
        <button class="ui fluid blue button" type="submit">
            <i class="search icon"></i> {search_label}
        </button>
    """
    if hasattr(cl.model_admin, "filterset"):
        html = f"""
            <div class="field">
                {BLANK_LABEL}{search_button}
            </div>
        """
    else:
        html = FIELD.format(search_button)
    return html


@register.simple_tag(takes_context=True)
def search_fields(context, cl):
    html = ""
    search_field = format_search_field(context, cl)
    if hasattr(cl.model_admin, "filterset"):
        fields = [search_field]
        filter_field = """
            <label for="{field_id}">{label}: </label>
            {field}{errors}
        """
        filterset = cl.model_admin.filterset
        form = filterset.form
        for field in filterset.form:
            # WTF
            label = _(field.label.lower()).capitalize()
            if isinstance(form.fields[field.name].widget, forms.HiddenInput):
                f = f"""
                    <label for="{field.id_for_label}">{label}: </label>
                    <strong>{filterset.email}</strong>
                    {field}{field.errors}
                """
            else:
                format_dict = dict(
                    field_id=field.id_for_label,
                    label=label,
                    field=field,
                    errors=field.errors,
                )
                f = filter_field.format(**format_dict)
            f = FIELD.format(f)
            fields.append(f)
        try:
            from semantic_admin.filters import SemanticExcludeAllFilterSet
        except ImportError:
            pass
        else:
            if isinstance(cl.model_admin.filterset, SemanticExcludeAllFilterSet):
                exclude_label = _("Exclude")
                checked = cl.model_admin.filterset_exclude
                exclude_checkbox = f"""
                    {BLANK_LABEL}
                    <div class="ui checkbox">
                        <input
                            id="exclude"
                            type="checkbox"
                            tabindex="0"
                            class="hidden"
                            name="_exclude"
                            value="true"
                            {checked}
                        >
                        <label for="exclude">{exclude_label}</label>
                    </div>
                """
                f = FIELD.format(exclude_checkbox)
                fields.append(f)
        html += format_fields(cl, fields)
    else:
        html = search_field
    return format_html(mark_safe(html))



from django import forms
from django.contrib.admin.widgets import AutocompleteMixin


class SemanticAutocompleteMixin(AutocompleteMixin):
    """
    Select widget mixin that loads options from SemanticAutocompleteJsonView
    via AJAX. Renders the necessary data attributes for Semantic UI Dropdowns
    and adds the static form media.
    """

    template_name = "semantic_ui/forms/widgets/chooser.html"

    @property
    def media(self):
        # No media.
        pass


class SemanticAutocompleteSelect(  # type: ignore
    SemanticAutocompleteMixin, forms.Select
):
    template_name = "semantic_ui/forms/widgets/select.html"


class SemanticAutocompleteSelectMultiple(  # type: ignore
    SemanticAutocompleteMixin, forms.SelectMultiple
):
    template_name = "semantic_ui/forms/widgets/select.html"



from .filters import (
    SemanticChoiceFilter,
    SemanticDateFilter,
    SemanticDateTimeFilter,
    SemanticModelChoiceFilter,
    SemanticModelMultipleChoiceFilter,
    SemanticMultipleAllValuesFilter,
    SemanticMultipleChoiceFilter,
    SemanticTimeFilter,
    SemanticTypedChoiceFilter,
    SemanticTypedMultipleChoiceFilter,
)
from .filterset import SemanticExcludeAllFilterSet, SemanticFilterSet

__all__ = [
    "SemanticExcludeAllFilterSet",
    "SemanticFilterSet",
    "SemanticDateTimeFilter",
    "SemanticDateFilter",
    "SemanticTimeFilter",
    "SemanticChoiceFilter",
    "SemanticMultipleChoiceFilter",
    "SemanticTypedChoiceFilter",
    "SemanticTypedMultipleChoiceFilter",
    "SemanticMultipleAllValuesFilter",
    "SemanticModelChoiceFilter",
    "SemanticModelMultipleChoiceFilter",
]



import datetime
import decimal
import json

from django.contrib.admin.utils import display_for_value
from django.db import models
from django.utils import formats, timezone
from django.utils.html import format_html

try:
    from django.contrib.postgres.fields import JSONField
except ImportError:
    from django.db.models import JSONField  # type: ignore


def semantic_display_for_field(value, field, empty_value_display):
    from .templatetags.semantic_admin_list import _semantic_boolean_icon

    if getattr(field, "flatchoices", None):
        return dict(field.flatchoices).get(value, empty_value_display)
    # BooleanField needs special-case null-handling, so it comes before the
    # general null test.

    # BEGIN CUSTOMIZATION
    elif isinstance(field, models.BooleanField):
        return _semantic_boolean_icon(value)
    # END CUSTOMIZATION

    elif value is None:
        return empty_value_display
    elif isinstance(field, models.DateTimeField):
        return formats.localize(timezone.template_localtime(value))
    elif isinstance(field, (models.DateField, models.TimeField)):
        return formats.localize(value)
    elif isinstance(field, models.DecimalField):
        return formats.number_format(value, field.decimal_places)
    elif isinstance(field, (models.IntegerField, models.FloatField)):
        return formats.number_format(value)
    elif isinstance(field, models.FileField) and value:
        return format_html('<a href="{}">{}</a>', value.url, value)
    elif isinstance(field, JSONField) and value:
        try:
            return json.dumps(value, ensure_ascii=False, cls=field.encoder)
        except TypeError:
            return display_for_value(value, empty_value_display)
    else:
        return display_for_value(value, empty_value_display)


def semantic_display_for_value(value, empty_value_display, boolean=False):
    from .templatetags.semantic_admin_list import _semantic_boolean_icon

    # BEGIN CUSTOMIZATION
    if boolean:
        return _semantic_boolean_icon(value)
    # END CUSTOMIZATION

    elif value is None:
        return empty_value_display
    elif isinstance(value, bool):
        return str(value)
    elif isinstance(value, datetime.datetime):
        return formats.localize(timezone.template_localtime(value))
    elif isinstance(value, (datetime.date, datetime.time)):
        return formats.localize(value)
    elif isinstance(value, (int, decimal.Decimal, float)):
        return formats.number_format(value)
    elif isinstance(value, (list, tuple)):
        return ", ".join(str(v) for v in value)
    else:
        return str(value)



import types

from django.template.library import Library
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe

register = Library()


@register.filter(is_safe=True, needs_autoescape=True)
def semantic_unordered_list(value, autoescape=True):
    """
    Recursively take a self-nested list and return an HTML unordered list --
    WITHOUT opening and closing <ul> tags.
    Assume the list is in the proper format. For example, if ``var`` contains:
    ``['States', ['Kansas', ['Lawrence', 'Topeka'], 'Illinois']]``, then
    ``{{ var|unordered_list }}`` returns::
        <div class="item">States
        <div class="ui bulleted list">
                <div class="item">Kansas</div>
                <div class="ui bulleted list">
                        <div class="item">Lawrence</div>
                        <div class="item">Topeka</div>
                </div>
                </div>
                <div class="item">Illinois</div>
        </div>
        </div>
    """
    if autoescape:
        escaper = conditional_escape
    else:

        def escaper(x):
            return x

    def walk_items(item_list):
        item_iterator = iter(item_list)
        try:
            item = next(item_iterator)
            while True:
                try:
                    next_item = next(item_iterator)
                except StopIteration:
                    yield item, None
                    break
                if isinstance(next_item, (list, tuple, types.GeneratorType)):
                    try:
                        iter(next_item)
                    except TypeError:
                        pass
                    else:
                        yield item, next_item
                        item = next(item_iterator)
                        continue
                yield item, None
                item = next_item
        except StopIteration:
            pass

    def list_formatter(item_list, tabs=1):
        indent = "\t" * tabs
        output = []
        for item, children in walk_items(item_list):
            sublist = ""
            if children:
                sublist = '\n%s<div class="ui bulleted list">\n%s\n%s</div>\n%s' % (
                    indent,
                    list_formatter(children, tabs + 1),
                    indent,
                    indent,
                )
            output.append(
                '%s<div class="item">%s%s</div>' % (indent, escaper(item), sublist)
            )
        return "\n".join(output)

    return mark_safe(list_formatter(value))



from django.contrib import admin
from django.contrib.admin.views.main import ChangeList
from django.forms import ModelMultipleChoiceField
from django.forms.fields import MultipleChoiceField
from django.utils.http import urlencode


class AwesomeSearchChangeList(ChangeList):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        try:
            request = args[0]
        except IndexError:
            pass
        else:
            # WTF? Wasn't this already set.
            self.full_result_count = self.model_admin.get_queryset(request).count()

    def get_query_string(self, new_params=None, remove=None):
        params = super().get_query_string(new_params=new_params, remove=remove)
        filterset_params = self.get_filterset_params()
        if filterset_params:
            if len(params) > 1:
                params += "&"
            params += filterset_params
        return params

    def get_filterset_params(self):
        filterset_params = getattr(self.model_admin, "filterset_params", None)
        params = ""
        if filterset_params:
            for key in filterset_params:
                value = filterset_params[key]
                if isinstance(value, list):
                    if len(value) == 1:
                        param = urlencode({key: value[0]})
                        params += param + "&"
                    else:
                        for v in value:
                            param = urlencode({key: v})
                            params += param + "&"
                else:
                    param = urlencode({key: value})
                    params += param + "&"
            params = params[:-1]  # Remove the last '&'
        return params


class AwesomeSearchModelAdmin(admin.ModelAdmin):
    """
    Subclass based on https://djangosnippets.org/snippets/2322/ allowing
    advanced filtering with https://github.com/alex/django-filter
    """

    def get_changelist(self, request, **kwargs):
        """Returns the ChangeList class for use on the changelist page."""
        return AwesomeSearchChangeList

    def changelist_view(self, request, extra_context=None, **kwargs):
        """
        FilterSet params must be removed from request within this function, or
        IncorrectLookupParameters exception will be raised.
        """
        # Copy request.GET for preserved filters.
        self.awesome_preserved_filters = request.GET.copy()
        # To remove filterset params from request, temporarily make
        # request.GET mutable
        request.GET._mutable = True
        if hasattr(self, "filter_class"):
            # Reset filterset params every request
            self.get_filterset_params(request)
        # Restore immutability of request.GET
        request.GET._mutable = False
        return super().changelist_view(request, extra_context=extra_context)

    def get_preserved_filters(self, request):
        """
        Return the preserved filters querystring.
        """
        match = request.resolver_match
        if self.preserve_filters and match:
            opts = self.model._meta
            current_url = "%s:%s" % (match.app_name, match.url_name)
            changelist_url = "admin:%s_%s_changelist" % (
                opts.app_label,
                opts.model_name,
            )
            if current_url == changelist_url:
                # BEGIN CUSTOMIZATION #
                preserved_filters = self.awesome_preserved_filters.urlencode()
                # END CUSTOMIZATION #
            else:
                preserved_filters = request.GET.get("_changelist_filters")

            if preserved_filters:
                return urlencode({"_changelist_filters": preserved_filters})
        return ""

    def get_filterset_params(self, request):
        if hasattr(self, "filter_class"):
            # Does this matter?
            filter_class = self.filter_class(request=request)
            if hasattr(filter_class, "get_initial"):
                self.filterset_params = filter_class.get_initial()
            else:
                self.filterset_params = {}
            form = filter_class.form
            for field in form.fields:
                # Is it a RangeField, etc?
                if hasattr(form.fields[field], "fields"):
                    for index, _ in enumerate(form.fields[field].fields):
                        param = "{}_{}".format(field, index)
                        self.set_filterset_param(request, form, field, param)
                else:
                    param = field
                    self.set_filterset_param(request, form, field, param)

    def set_filterset_param(self, request, form, field, param):
        is_multiple = isinstance(form.fields[field], MultipleChoiceField) or isinstance(
            form.fields[field], ModelMultipleChoiceField
        )
        # Get list for multiple, or single value
        if is_multiple:
            value = request.GET.getlist(param)
            value = [v for v in value if v != ""]  # No blank params
        else:
            value = request.GET.get(param, None)
        # Set value
        if value:
            self.filterset_params[param] = value
        # Delete param every request
        if param in request.GET:
            del request.GET[param]

    def get_search_results(self, request, queryset, search_term):
        """
        Returns a tuple containing a queryset to implement the search,
        and a boolean indicating if the results may contain duplicates.
        """
        queryset, may_have_duplicates = super().get_search_results(
            request, queryset, search_term
        )
        if hasattr(self, "filter_class") and hasattr(self, "filterset_params"):
            kwargs = {
                "request": request,
                "queryset": queryset,
                "passed_validation": True,
            }
            self.filterset = filterset = self.filter_class(
                self.filterset_params, **kwargs
            )
            queryset = filterset.qs
        return queryset, may_have_duplicates



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['semantic_admin',
 'semantic_admin.filters',
 'semantic_admin.templatetags',
 'semantic_admin.views',
 'semantic_admin.widgets']

package_data = \
{'': ['*'],
 'semantic_admin': ['static/admin/js/*',
                    'static/admin/js/admin/*',
                    'static/semantic_admin/*',
                    'templates/admin/*',
                    'templates/admin/auth/user/*',
                    'templates/admin/edit_inline/*',
                    'templates/admin/includes/*',
                    'templates/admin/widgets/*',
                    'templates/registration/*',
                    'templates/semantic_ui/forms/widgets/*']}

install_requires = \
['django>=3.2']

setup_kwargs = {
    'name': 'django-semantic-admin',
    'version': '0.3.4',
    'description': 'Django Semantic UI Admin theme',
    'long_description': 'Django Semantic UI admin theme\n------------------------------\n<img src="https://raw.githubusercontent.com/globophobe/django-semantic-admin/master/docs/screenshots/change-list.png" alt="django-semantic-admin"/>\n\nA completely free (MIT) [Semantic UI](https://semantic-ui.com/) admin theme for Django. Actually, this is my 3rd admin theme for Django. The first was forgettable, and the second was with [Pure CSS](https://purecss.io/). Pure CSS was great, but lacked JavaScript components.\n\nSemantic UI looks professional, and has great JavaScript components.\n\nLog in to the demo with username `django` and password `semantic-admin`: https://semantic-admin.com\n\nDocumentation is on [GitHub Pages](https://globophobe.github.io/django-semantic-admin/).\n\nWhy?\n----\n* Looks professional, with a nice sidebar.\n* Responsive design, even [tables can stack](https://semantic-ui.com/collections/table.html#stacking) responsively on mobile.\n* JavaScript datepicker and timepicker components.\n* JavaScript selects, including multiple selections, which integrate well with Django autocomplete fields.\n* Semantic UI has libraries for [React](https://react.semantic-ui.com/) and [Vue](https://semantic-ui-vue.github.io/#/), in addition to jQuery. This means this package can be used to style the admin, and custom views can be added with React or Vue components with the same style.\n\n\nInstall\n-------\n\nInstall from PyPI:\n\n```\npip install django-semantic-admin\n```\n\nAdd to `settings.py` before `django.contrib.admin`:\n\n```python\nINSTALLED_APPS = [\n    "semantic_admin",\n    "django.contrib.admin",\n    ...\n]\n```\n\nUsage\n-----\n\nInstead of `admin.ModelAdmin`, `admin.StackedInline`, or `admin.TabularInline`:\n\n```python\nclass ExampleStackedInline(admin.StackedInline):\n    pass\n\nclass ExampleTabularInline(admin.TabularInline):\n    pass\n\nclass ExampleAdmin(admin.ModelAdmin):\n    inlines = (ExampleStackedInline, ExampleTabularInline)\n```\n\nInherit from their `Semantic` equivalents:\n\n```python\nfrom semantic_admin import SemanticModelAdmin, SemanticStackedInline, SemanticTabularInline\n\nclass ExampleStackedInline(SemanticStackedInline):\n    pass\n\nclass ExampleTabularInline(SemanticTabularInline):\n    pass\n\nclass ExampleAdmin(SemanticModelAdmin):\n    inlines = (ExampleStackedInline, ExampleTabularInline)\n```\n\nAwesome optional features\n-------------------------\n\n1. Optional integration with [django_filter](https://github.com/carltongibson/django-filter):\n\n<img src="https://raw.githubusercontent.com/globophobe/django-semantic-admin/master/docs/screenshots/django-filter.png" width="335" alt="django-filter" />\n\nTo enable this awesome feature, add `filter_class` to your Django admin:\n\n```python\nfrom semantic_admin.filters import SemanticFilterSet\n\nclass DemoFilter(SemanticFilterSet):\n    class Meta:\n        model = Demo\n        fields = ("demo_field",)\n\nclass DemoAdmin(SemanticModelAdmin):\n    filter_class = DemoFilter\n```\n\n2. HTML preview in Django `autocomplete_fields`:\n\n<img src="https://raw.githubusercontent.com/globophobe/django-semantic-admin/master/docs/screenshots/html5-autocomplete.png" width="670" alt="html5-autocomplete" />\n\nTo enable this awesome feature, add the `semantic_autocomplete` property to your Django model:\n\n```python\nclass DemoModel(models.Model):\n    @property\n    def semantic_autocomplete(self):\n        html = self.get_img()\n        return format_html(html)\n```\n\nContributing\n------------\n\nInstall dependencies with `poetry install`. The demo is built with [invoke tasks](https://github.com/globophobe/django-semantic-admin/blob/master/demo/tasks.py). For example, `cd demo; invoke build`.\n\n\nNotes\n-----\nPlease note, this package uses [Fomantic UI](https://fomantic-ui.com/) the official community fork of Semantic UI.\n',
    'author': 'Alex',
    'author_email': 'globophobe@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/globophobe/django-semantic-admin',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import datetime

from django import template
from django.contrib.admin.templatetags.admin_list import (
    _coerce_field_name,
    result_hidden_fields,
)
from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
from django.contrib.admin.utils import (
    display_for_field,
    display_for_value,
    label_for_field,
    lookup_field,
)
from django.contrib.admin.views.main import ORDER_VAR, PAGE_VAR
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.urls import NoReverseMatch, reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe

from .semantic_admin_list import semantic_results

register = template.Library()


def items_for_result(cl, result, form):
    """
    Generate the actual list of data.
    """

    def link_in_col(is_first, field_name, cl):
        if cl.list_display_links is None:
            return False
        if is_first and not cl.list_display_links:
            return True
        return field_name in cl.list_display_links

    first = True
    pk = cl.lookup_opts.pk.attname
    for field_index, field_name in enumerate(cl.list_display):
        empty_value_display = cl.model_admin.get_empty_value_display()
        row_classes = ["field-%s" % _coerce_field_name(field_name, field_index)]
        try:
            f, attr, value = lookup_field(field_name, result, cl.model_admin)
        except ObjectDoesNotExist:
            result_repr = empty_value_display
        else:
            empty_value_display = getattr(
                attr, "empty_value_display", empty_value_display
            )
            if f is None or f.auto_created:
                if field_name == "action_checkbox":
                    row_classes = ["action-checkbox"]
                boolean = getattr(attr, "boolean", False)
                result_repr = display_for_value(value, empty_value_display, boolean)
                if isinstance(value, (datetime.date, datetime.time)):
                    row_classes.append("nowrap")
            else:
                if isinstance(f.remote_field, models.ManyToOneRel):
                    field_val = getattr(result, f.name)
                    if field_val is None:
                        result_repr = empty_value_display
                    else:
                        result_repr = field_val
                else:
                    result_repr = display_for_field(value, f, empty_value_display)
                if isinstance(
                    f, (models.DateField, models.TimeField, models.ForeignKey)
                ):
                    row_classes.append("nowrap")

        # BEGIN CUSTOMIZATION #
        # WTF Unicode?
        if not result_repr:
            if str(result_repr) == "":
                result_repr = mark_safe("&nbsp;")
        # END CUSTOMIZATION #

        row_class = mark_safe(' class="%s"' % " ".join(row_classes))
        # If list_display_links not defined, add the link tag to the first
        # field
        if link_in_col(first, field_name, cl):

            # BEGIN CUSTOMIZATION #
            table_tag = "td"  # if first else 'td'
            # END CUSTOMIZATION #

            first = False

            # Display link to the result's change_view if the url exists, else
            # display just the result's representation.
            try:
                url = cl.url_for_result(result)
            except NoReverseMatch:
                link_or_text = result_repr
            else:
                url = add_preserved_filters(
                    {"preserved_filters": cl.preserved_filters, "opts": cl.opts}, url
                )
                # Convert the pk to something that can be used in Javascript.
                # Problem cases are non-ASCII strings.
                if cl.to_field:
                    attr = str(cl.to_field)
                else:
                    attr = pk
                value = result.serializable_value(attr)
                link_or_text = format_html(
                    '<a href="{}"{}>{}</a>',
                    url,
                    format_html(' data-popup-opener="{}"', value)
                    if cl.is_popup
                    else "",
                    result_repr,
                )

            yield format_html(
                "<{}{}>{}</{}>", table_tag, row_class, link_or_text, table_tag
            )
        else:
            # By default the fields come from ModelAdmin.list_editable, but if
            # we pull the fields out of the form instead of list_editable
            # custom admins can provide fields on a per request basis
            if (
                form
                and field_name in form.fields
                and not (
                    field_name == cl.model._meta.pk.name
                    and form[cl.model._meta.pk.name].is_hidden
                )
            ):
                bf = form[field_name]
                result_repr = mark_safe(str(bf.errors) + str(bf))
            yield format_html("<td{}>{}</td>", row_class, result_repr)
    if form and not form[cl.model._meta.pk.name].is_hidden:
        yield format_html("<td>{}</td>", form[cl.model._meta.pk.name])


def has_action_checkbox(cl):
    for i, field_name in enumerate(cl.list_display):
        text, attr = label_for_field(
            field_name, cl.model, model_admin=cl.model_admin, return_attr=True
        )
        if attr:
            field_name = _coerce_field_name(field_name, i)
            # Potentially not sortable

            # if the field is the action checkbox: no sorting and special class
            if field_name == "action_checkbox":
                return True


def result_headers(cl):
    """
    Generate the list column headers.
    """
    ordering_field_columns = cl.get_ordering_field_columns()
    for i, field_name in enumerate(cl.list_display):
        text, attr = label_for_field(
            field_name, cl.model, model_admin=cl.model_admin, return_attr=True
        )
        if attr:
            field_name = _coerce_field_name(field_name, i)
            # Potentially not sortable

            # if the field is the action checkbox: no sorting and special class
            if field_name == "action_checkbox":
                yield {
                    "text": mark_safe(text),
                    "class_attrib": mark_safe("action-checkbox-column"),
                    "sortable": False,
                }
                continue

            admin_order_field = getattr(attr, "admin_order_field", None)
            if not admin_order_field:
                # Not sortable
                yield {
                    "text": text,
                    "class_attrib": format_html("column-{}", field_name),
                    "sortable": False,
                }
                continue

        # OK, it is sortable if we got this far
        th_classes = ["sortable", "column-{}".format(field_name)]
        order_type = ""
        new_order_type = "asc"
        sort_priority = 0
        sorted = False
        # Is it currently being sorted on?
        if i in ordering_field_columns:
            sorted = True
            order_type = ordering_field_columns.get(i).lower()
            sort_priority = list(ordering_field_columns).index(i) + 1
            th_classes.append("sorted %sending" % order_type)
            new_order_type = {"asc": "desc", "desc": "asc"}[order_type]

        # build new ordering param
        o_list_primary = []  # URL for making this field the primary sort
        o_list_remove = []  # URL for removing this field from sort
        o_list_toggle = []  # URL for toggling order type for this field

        def make_qs_param(t, n):
            return ("-" if t == "desc" else "") + str(n)

        for j, ot in ordering_field_columns.items():
            if j == i:  # Same column
                param = make_qs_param(new_order_type, j)
                # We want clicking on this header to bring the ordering to the
                # front
                o_list_primary.insert(0, param)
                o_list_toggle.append(param)
                # o_list_remove - omit
            else:
                param = make_qs_param(ot, j)
                o_list_primary.append(param)
                o_list_toggle.append(param)
                o_list_remove.append(param)

        if i not in ordering_field_columns:
            o_list_primary.insert(0, make_qs_param(new_order_type, i))

        yield {
            "text": text,
            "sortable": True,
            "sorted": sorted,
            "ascending": order_type == "asc",
            "sort_priority": sort_priority,
            "url_primary": cl.get_query_string({ORDER_VAR: ".".join(o_list_primary)}),
            "url_remove": cl.get_query_string({ORDER_VAR: ".".join(o_list_remove)}),
            "url_toggle": cl.get_query_string({ORDER_VAR: ".".join(o_list_toggle)}),
            "class_attrib": format_html("{}", " ".join(th_classes))
            if th_classes
            else "",
        }


@register.inclusion_tag("admin/change_list_results.html")
def semantic_result_list(cl):
    """
    Display the headers and data list together.
    """
    headers = list(result_headers(cl))
    num_sorted_fields = 0
    for h in headers:
        if h["sortable"] and h["sorted"]:
            num_sorted_fields += 1
    return {
        "cl": cl,
        "opts": cl.model_admin.opts,  # WTF?!
        "has_action_checkbox": has_action_checkbox(cl),
        "result_hidden_fields": list(result_hidden_fields(cl)),
        "result_headers": headers,
        "num_sorted_fields": num_sorted_fields,
        "results": list(semantic_results(cl)),
    }


@register.simple_tag
def semantic_paginator_number(cl, i):
    """
    Generate an individual page index link in a paginated list.
    """
    DOT = "."
    ELLIPSIS = getattr(cl.paginator, "ELLIPSIS", None)
    if i == DOT or i == ELLIPSIS:
        return format_html('<span class="item">... </span>')
    elif i == cl.page_num:
        return format_html('<span class="this-page item active">{}</span> ', i)
    else:
        return format_html(
            '<a href="{}" class="item{}">{}</a> ',
            cl.get_query_string({PAGE_VAR: i}),
            mark_safe(" end" if i == cl.paginator.num_pages else ""),
            i,
        )


@register.simple_tag
def has_url(url):
    """Does the url exist?"""
    try:
        reverse(url)
    except NoReverseMatch:
        return False
    else:
        return True
